//  Mugshot
//
//  Created by Blake Seely on 9/13/05.
//  Copyright __MyCompanyName__ 2005 . All rights reserved.

#import <Cocoa/Cocoa.h>
#import "MUDownloadController.h"
#import "MUModelClasses.h"
#import "MUFlickrHandlers.h"
#import "MUPhoto.h"
#import "RBSplitView.h"

@interface MugshotAppDelegate : NSObject 
{    
    NSManagedObjectModel *managedObjectModel;
    NSManagedObjectContext *managedObjectContext;
	
	NSManagedObject *contactCollection;
	NSManagedObject *groupCollection;
	NSManagedObject *photosetCollection;
	NSManagedObject *favoriteCollection;
	NSManagedObject *photostream;
	
	NSString *flickrToken;
	NSString *flickrFrob;
	NSDictionary *serviceLibrary;
	MUFlickrCacheHandler *cacheHandler;
	
	// New Mugshot Window
	IBOutlet RBSplitView *collectionSplitView;
	IBOutlet RBSplitView *photoSplitView;
	IBOutlet NSTableView *flickrTableView;
	IBOutlet NSArrayController *flickrArrayController;
	IBOutlet NSArrayController *flickrPhotos;
	IBOutlet NSTableView *albumsTableView;
	IBOutlet NSArrayController *albumsArrayController;
	IBOutlet NSArrayController *albumPhotos;
	IBOutlet NSTableView *contactsTableView;
	IBOutlet NSArrayController *contactsArrayController;
	IBOutlet NSArrayController *contactPhotos;
	IBOutlet NSTableView *groupsTableView;
	IBOutlet NSArrayController *groupsArrayController;
	IBOutlet NSArrayController *groupPhotos;
	NSArrayController *currentPhotosArrayController;
    NSMutableDictionary *currentPhotosCache;
	MUCollection *selectedCollection;
	IBOutlet NSProgressIndicator *statusSpinner;
	IBOutlet NSTextField *statusMessage;
	
	// Authorization
	IBOutlet NSPanel *authorizationStartPanel;
	IBOutlet NSButton *authorizeButton;
	IBOutlet NSPanel *authorizationFinishPanel;
	
	IBOutlet NSWindow *splashWindow;
	IBOutlet NSTextField *splashStatus;
	IBOutlet NSProgressIndicator *splashProgress;
	
	IBOutlet NSWindow *mugshotWindow;
	IBOutlet NSView *photoView;
	IBOutlet NSScrollView *photoScrollView;
	
	IBOutlet NSPanel *debugPhotoPanel;
	
	IBOutlet MUDownloadController *downloadController;
	BOOL isIPhotoDownload;
	
	NSAlert *errorPanel;
	NSMutableArray *pendingErrors;
    
    // MUPhotoView support
    NSMutableDictionary *currentNSImages;
    NSImage *loadingImage;
	
}

- (NSManagedObjectModel *)managedObjectModel;
- (NSManagedObjectContext *)managedObjectContext;

- (IBAction)saveAction:(id)sender;
- (IBAction)sendFeedback:(id)sender;

// New Mugshot Window
- (IBAction)shouldShowOrHideSplit:(id)sender;

// Authorization and Startup Actions
- (IBAction)doQuit:sender;
- (void)startupGetFrob;
- (IBAction)doAuthorizationLaunch:sender;
- (IBAction)doAuthorizationFinish:sender;
- (void)doMugshotStartup;

// Download
- (IBAction)downloadSelectedPhotosToDisk:(id)sender;
- (IBAction)downloadSelectedPhotosToiPhoto:(id)sender;
- (void)didDownloadSelectedPhoto:(NSManagedObject *)photo toPath:(NSString *)path;
- (void)didDownloadSelectedPhotos:(NSArray *)photos toPaths:(NSArray *)paths;
- (NSString *)fileCommentsForPhoto:(NSManagedObject *)photo;

// Accessors
- (NSString *)flickrToken;
- (void)setFlickrToken:(NSString *)token;
- (NSString *)flickrFrob;
- (void)setFlickrFrob:(NSString *)frob;
- (NSManagedObject *)contactCollection;
- (NSManagedObject *)groupCollection;
- (NSManagedObject *)photosetCollection;
- (NSManagedObject *)favoriteCollection;
- (NSManagedObject *)photostream;
- (NSWindow *)currentWindow;

// Utilities
- (void)flickrAddPhotos:(NSSet *)validPhotos toCollection:(MUCollection *)collection;
- (IBAction)removeSelectedPhotos;
- (void)flickrRemovePhotos:(NSSet *)validRemovals fromCollection:(MUCollection *)collection;
- (BOOL)shouldUpdatePhotosSet:(NSMutableSet *)photosSet withPhotoData:(NSArray *)photoDicts fromPageNumber:(int)page withPhotosPerPage:(int)perPage withNumberOfPhotos:(int)totalPhotos;
- (void)didCachePhotoNumber:(int)photoNumber outOfTotalNumber:(int)totalPhotos;
- (void)loadChildren:(NSArray *)childrenDicts toSet:(NSMutableSet *)childrenSet withEntityName:(NSString *)name;
- (void)addPhotosToMutableSet:(NSMutableSet *)set fromFlickrResults:(NSDictionary *)results usingResponseKeyPath:(NSString *)keyPath;
- (void)photoSelectionDidChange;
- (void)handlerError:(NSDictionary *)errorDict;
- (void)setInfo:(NSDictionary *)resultDict forPhoto:(MUPhoto *)photo;
- (void)removeCollectionItems;
- (IBAction)showDebugPhotoPanel:(id)sender;


// FlickrHandler Delegate Methods
- (void)finishContactsGetList:(NSDictionary *)resultDict;
- (void)finishPeopleGetInfo:(NSDictionary *)resultDict;
- (void)finishGroupsPoolsGetGroups:(NSDictionary *)resultDict;
- (void)finishGroupsGetInfo:(NSDictionary *)resultDict;
- (void)finishPhotosetsGetList:(NSDictionary *)resultDict;
- (void)finishPhotosSearch:(NSDictionary *)resultDict;
- (void)finishPhotosGetSizes:(NSDictionary *)resultDict;
- (void)finishPhotosGetContactsPhotos:(NSDictionary *)resultDict;
- (void)finishFavoritesGetList:(NSDictionary *)resultDict;
- (void)finishPhotosGetInfo:(NSDictionary *)resultDict;
- (void)finishPhotosetsGetPhotos:(NSDictionary *)resultDict;
- (void)finishGroupsPoolsGetPhotos:(NSDictionary *)resultDict;


// Error Management
// testing
- (IBAction)testErrorHandling:(id)sender;
- (void)shouldHandleError:(NSError *)error;
- (void)shouldHandleErrorSeverity:(int)severity code:(int)code domain:(NSString *)domain message:(NSString *)message detailMessage:(NSString *)detailMessage;
- (void)alertDidEnd:(NSAlert *)alert returnCode:(int)returnCode  contextInfo:(void  *)contextInfo;
- (void)fatalAlertDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (void)logMessageSeverity:(NSString *)severity message:(NSString *)message detailMessage:(NSString *)detailMessage domain:(NSString *)domain;

@end
